unit Wizard97;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, WizConst, Wiz97Frm;

const
  INDEX_WELCOME    = 0;
  INDEX_INTERIOR   = 1;
  INDEX_COMPLETION = 2;

type
  TWizardPage = class;
  TWizardPages = class;
  TCustomWizard97 = class;

  // Enumerated types
  TExecuteResult = (erCanceled, erFinished);
  TWizardStyles = (wsModal, wsOnTop);
  TWizardStyle = set of TWizardStyles;
  TPageType = (ptWelcome, ptInterior, ptCompletion);

  // Event types
  TPageChangingEvent = procedure (Sender: TObject; Page: TWizardPage;
    var AllowChange: Boolean) of object;
  TPageChangeEvent = procedure (Sender: TObject; Page: TWizardPage) of object;
  TOnInitEvent = TNotifyEvent;
  TCancelEvent = TNotifyEvent;
  TCompletionEvent = TNotifyEvent;

  // Exception types
  EDuplicatePageName = class(Exception);

  TWizardPage = class(TPersistent)
  private
    FOwner: TWizardPages;
    FInternalIndex: Integer;
    FPageName: string;
    FPageType: TPageType;
    procedure SetPageName(Value: string); virtual;
    procedure BtnBackClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnFinishClick(Sender: TObject);
    procedure BtnNextClick(Sender: TObject);
    procedure SetButton(Button: TButton; const Caption: TCaption;
      const Enabled: Boolean; OnClick: TNotifyEvent);
  public
    constructor Create(AOwner: TWizardPages; PageName: string); virtual;
    destructor Destroy; override;
    procedure Initialize; virtual;
    property Pages: TWizardPages read FOwner;
    property PageName: string read FPageName write SetPageName;
    property PageType: TPageType read FPageType;
  end;

  TWelcomePage = class(TWizardPage)
  private
    FShowContinue: Boolean;
  public
    constructor Create(AOwner: TWizardPages; PageName: string); override;
    procedure Initialize; override;
  published
    property ShowContinue: Boolean read FShowContinue write FShowContinue;
  end;

  TInteriorPage = class(TWizardPage)
  public
    constructor Create(AOwner: TWizardPages; PageName: string); override;
    procedure Initialize; override;
  end;

  TCompletionPage = class(TWizardPage)
  public
    constructor Create(AOwner: TWizardPages; PageName: string); override;
    procedure Initialize; override;
  end;

  TWizardPages = class(TPersistent)
  private
    FOwner: TCustomWizard97;
  protected
    FPageList: TStringList;
    function GetCount: Integer;
    function GetItem(Index: Integer): TWizardPage;
  public
    constructor Create(AOwner: TCustomWizard97);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure DeletePage(Index: Integer);
    procedure Empty;
    function AddPage(InteriorPage: TInteriorPage): Integer;
    property Page[Index: Integer]: TWizardPage read GetItem; default;
    property PageCount: Integer read GetCount;
  end;

  TCustomWizard97 = class(TComponent)
  private
    FExecuting: Boolean;
    FFormCaption: TCaption;
    FPages: TWizardPages;
    FPageIndex: Integer;
    FOnPageChanging: TPageChangingEvent;
    FOnPageChange: TPageChangeEvent;
    FOnInit: TOnInitEvent;
    FOnCancel: TCancelEvent;
    FOnClose: TNotifyEvent;
    FOnCompletion: TCompletionEvent;
    FOnNextPage: TNotifyEvent;
    FOnPrevPage: TNotifyEvent;
    FStyle: TWizardStyle;
    FSymbolGraphic: TBitmap;
    FWizard97Form: TWizard97Form;
    FWizardName: string;
    procedure DoCancel;
    procedure DoClose;
    procedure DoBackClick(Sender: TObject);
    procedure DoNextClick(Sender: TObject);
    procedure DoCancelClick(Sender: TObject);
    procedure DoCompletion;
    procedure DoInitialize;
    procedure SetCaption(Value: TCaption);
    procedure SetPages(Value: TWizardPages);
    procedure SetWizardName(Value: string);
    procedure SetSymbolGraphic(Value: TBitmap);
    function WizardForm: TWizard97Form;
    procedure WizardFormClose(Sender: TObject; var Action: TCloseAction);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function Execute: TExecuteResult;
    function Executing: Boolean;
    procedure NextPage;
    procedure PrevPage;
    property FormCaption: TCaption read FFormCaption write SetCaption;
    property Pages: TWizardPages read FPages write SetPages;
    property Style: TWizardStyle read FStyle write FStyle;
    property SymbolGraphic: TBitmap read FSymbolGraphic write SetSymbolGraphic;
    property WizardName: string read FWizardName write SetWizardName;
    property OnCancel: TCancelEvent read FOnCancel write FOnCancel;
    property OnCompletion: TCompletionEvent read FOnCompletion write FOnCompletion;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnInit: TOnInitEvent read FOnInit write FOnInit;
    property OnNextPage: TNotifyEvent read FOnNextPage write FOnNextPage;
    property OnPageChange: TPageChangeEvent read FOnPageChange write
      FOnPageChange;
    property OnPageChanging: TPageChangingEvent read FOnPageChanging write
      FOnPageChanging;
    property OnPrevPage: TNotifyEvent read FOnPrevPage write FOnPrevPage;
  end;

  TWizard97 = class(TCustomWizard97)
  published
    property FormCaption;
    property WizardName;
    property Pages;
    property Style;
    property SymbolGraphic;
    property OnInit;
    property OnCancel;
    property OnClose;
    property OnCompletion;
    property OnNextPage;
    property OnPageChanging;
    property OnPageChange;
    property OnPrevPage;
  end;

implementation

{$R *.dcr}
{$R WIZARD97.RES}

procedure ValidateString(var S: string);
begin
  if Trim(S) = EMPTYSTRING then
    S := DEFAULTWIZARDNAME;
end;

// TWizardPage
constructor TWizardPage.Create(AOwner: TWizardPages; PageName: string);
begin
  inherited Create;
  FOwner := AOwner;
  FInternalIndex := AOwner.GetCount;
  FPageName := PageName;
end;

destructor TWizardPage.Destroy;
begin
  if Assigned(FOwner) then
    FOwner.DeletePage(FInternalIndex);
end;

procedure TWizardPage.BtnBackClick(Sender: TObject);
var
  AllowChange: Boolean;
  Wizard: TCustomWizard97;
  NewPage: TWizardPage;
begin
  Wizard := FOwner.FOwner;
  AllowChange := TRUE;
  if Wizard.Executing then
  begin
    NewPage := Pages.GetItem(Wizard.FPageIndex - 1);
    if Assigned(Wizard.FOnPageChanging) then
      Wizard.FOnPageChanging(Wizard, Self, AllowChange);
    if AllowChange then
    begin
      if Assigned(Wizard.FOnNextPage) then
        Wizard.FOnNextPage(Self);
      Wizard.PrevPage;
      if Assigned(Wizard.FOnPageChange) then
        Wizard.FOnPageChange(Wizard, NewPage);
    end;
  end;
end;

procedure TWizardPage.BtnCancelClick(Sender: TObject);
var
  Wizard: TCustomWizard97;
begin
  Wizard := FOwner.FOwner;
  with Wizard do
  if FExecuting then
  begin
    if MessageDlg(Format(MSG_CANCELEXECUTE, [WizardName]), mtConfirmation,
      [mbYes, mbNo], 0) = mrYes then
      DoCancel;
  end;
end;

procedure TWizardPage.BtnFinishClick(Sender: TObject);
var
  Wizard: TCustomWizard97;
begin
  Wizard := FOwner.FOwner;
  with Wizard do
    if FExecuting then DoCompletion;
end;

procedure TWizardPage.BtnNextClick(Sender: TObject);
var
  AllowChange: Boolean;
  Wizard: TCustomWizard97;
  NewPage: TWizardPage;
begin
  Wizard := FOwner.FOwner;
  AllowChange := TRUE;
  if Wizard.Executing then
  begin
    NewPage := Pages.GetItem(Wizard.FPageIndex + 1);
    if Assigned(Wizard.FOnPageChanging) then
      Wizard.FOnPageChanging(Wizard, Self, AllowChange);
    if AllowChange then
    begin
      if Assigned(Wizard.FOnNextPage) then
        Wizard.FOnNextPage(Self);
      Wizard.NextPage;
      if Assigned(Wizard.FOnPageChange) then
        Wizard.FOnPageChange(Wizard, NewPage);
    end;
  end;
end;

procedure TWizardPage.Initialize;
begin
  with FOwner.FOwner.FWizard97Form do
  begin
    SetButton(btnBack, TXT_BACK, FALSE, nil);
    SetButton(btnNext, TXT_NEXT, FALSE, nil);
    SetButton(btnCancel, TXT_CANCEL, FALSE, nil);
  end;
end;

procedure TWizardPage.SetButton(Button: TButton; const Caption: TCaption;
  const Enabled: Boolean; OnClick: TNotifyEvent);
begin
  Button.Caption := Caption;
  Button.Enabled := Enabled;
  Button.OnClick := OnClick;
end;

procedure TWizardPage.SetPageName(Value: string);
var
  NewPageName : string;
begin
  if FPageName <> Value then
  begin
    NewPageName := Value;
    ValidateString(NewPageName);
    if FOwner.FPageList.IndexOf(NewPageName) = -1 then
      FPageName := NewPageName
    else
      raise EDuplicatePageName.CreateFmt('Page %s already exists', [NewPageName]);
  end;
    FPageName := Value;
end;

// TWelcomePage
constructor TWelcomePage.Create(AOwner: TWizardPages; PageName: string);
begin
  inherited Create(AOwner, PageName);
  FPageType := ptWelcome;
end;

procedure TWelcomePage.Initialize;
begin
  with FOwner.FOwner.FWizard97Form do
  begin
    SetButton(btnBack, TXT_BACK, FALSE, nil);
    SetButton(btnNext, TXT_NEXT, TRUE, BtnNextClick);
    SetButton(btnCancel, TXT_CANCEL, TRUE, BtnCancelClick);
    nbkWizard.PageIndex := INDEX_WELCOME;
  end
end;

// TInteriorPage
constructor TInteriorPage.Create(AOwner: TWizardPages; PageName: string);
begin
  inherited Create(AOwner, PageName);
  FPageType := ptInterior;
end;

procedure TInteriorPage.Initialize;
begin
  with FOwner.FOwner.FWizard97Form do
  begin
    SetButton(btnBack, TXT_BACK, TRUE, BtnBackClick);
    SetButton(btnNext, TXT_NEXT, TRUE, BtnNextClick);
    SetButton(btnCancel, TXT_CANCEL, TRUE, BtnCancelClick);
    nbkWizard.PageIndex := INDEX_INTERIOR;
  end
end;

// TCompletionPage
constructor TCompletionPage.Create(AOwner: TWizardPages; PageName: string);
begin
  inherited Create(AOwner, PageName);
  FPageType := ptCompletion;
end;

procedure TCompletionPage.Initialize;
begin
  with FOwner.FOwner.FWizard97Form do
  begin
    SetButton(btnBack, TXT_BACK, TRUE, BtnBackClick);
    SetButton(btnNext, TXT_FINISH, TRUE, BtnFinishClick);
    SetButton(btnCancel, TXT_CANCEL, TRUE, BtnCancelClick);
    nbkWizard.PageIndex := INDEX_COMPLETION;
  end
end;

// TWizardPages
constructor TWizardPages.Create(AOwner: TCustomWizard97);
var
  WelcomePage: TWelcomePage;
  CompletionPage: TCompletionPage;
begin
  inherited Create;
  FOwner := AOwner;
  FPageList := TStringList.Create;
  WelcomePage := TWelcomePage.Create(Self, 'Welcome Page');
  FPageList.AddObject(WelcomePage.PageName, WelcomePage);
  CompletionPage := TCompletionPage.Create(Self, 'Completion Page');
  FPageList.AddObject(CompletionPage.PageName, CompletionPage);
end;

destructor TWizardPages.Destroy;
begin
  Empty;
  FPageList.Free;
end;

function TWizardPages.AddPage(InteriorPage: TInteriorPage): Integer;
begin
  Result := FPageList.Count - 1;
  FPageList.InsertObject(Result, InteriorPage.PageName, InteriorPage);
end;

procedure TWizardPages.Assign(Source: TPersistent);
var
  SourcePages: TWizardPages;
begin
  if Source is TWizardPages then
  begin
    SourcePages := Source as TWizardPages;
    FPageList.Assign(SourcePages.FPageList);
  end else
    inherited Assign(Source);
end;

procedure TWizardPages.DeletePage(Index: Integer);
begin
  FPageList.Delete(Index);
end;

procedure TWizardPages.Empty;
begin
  with FPageList do
    while Count > 0 do
    begin
      Objects[Count - 1].Free;
      Objects[Count - 1] := nil;
      Delete(Count - 1);
    end;
end;

function TWizardPages.GetCount: Integer;
begin
  Result := FPageList.Count;
end;

function TWizardPages.GetItem(Index: Integer): TWizardPage;
begin
  Result := TWizardPage(FPageList.Objects[Index]);
end;

constructor TCustomWizard97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWizardName := DEFAULTWIZARDNAME;
  FStyle := [];
  FPages := TWizardPages.Create(Self);
  FSymbolGraphic := TBitmap.Create;
  FExecuting := FALSE;
end;

destructor TCustomWizard97.Destroy;
begin
  FPages.Free;
  FSymbolGraphic.Free;
  if Assigned(FWizard97Form) then
    FWizard97Form.Free;
  inherited Destroy;
end;

procedure TCustomWizard97.Assign(Source: TPersistent);
var
  SourceWizard: TCustomWizard97;
begin
  if Source is TCustomWizard97 then
  begin
    SourceWizard := Source as TCustomWizard97;
    // Copy properties from SourceWizard
    FPages.Assign(SourceWizard.Pages);
    FFormCaption := SourceWizard.FormCaption;
    FWizardName := SourceWizard.WizardName;
    FSymbolGraphic.Assign(SourceWizard.SymbolGraphic);
    Exit;
  end;
  inherited Assign(Source);
end;

procedure TCustomWizard97.SetPages(Value: TWizardPages);
begin
  FPages.Assign(Value);
end;

procedure TCustomWizard97.WizardFormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  DoClose;
end;

procedure TCustomWizard97.DoCancel;
begin
  if Assigned(FOnCancel) then FOnCancel(Self);
  WizardForm.Close;
end;

procedure TCustomWizard97.DoClose;
begin
  FExecuting := FALSE;
  if Assigned(FOnClose) then FOnClose(Self);
end;

procedure TCustomWizard97.DoCompletion;
begin
  if Assigned(FOnCompletion) then FOnCompletion(Self);
  WizardForm.Close;
end;

procedure TCustomWizard97.DoBackClick(Sender: TObject);
begin
//  Pages[FPageIndex].BtnBackClick(Sender);
end;

procedure TCustomWizard97.DoNextClick(Sender: TObject);
begin
//  Pages[FPageIndex].BtnBackClick(Sender);
end;

procedure TCustomWizard97.DoCancelClick(Sender: TObject);
begin
//  Pages[FPageIndex].BtnBackClick(Sender);
end;

procedure TCustomWizard97.DoInitialize;
begin
  with WizardForm do
  begin
    imgInteriorSymbol.Picture.Bitmap.Assign(FSymbolGraphic);
    imgWelcomeSymbol.Picture.Bitmap.Assign(FSymbolGraphic);
    OnClose := WizardFormClose;
    btnBack.OnClick := DoBackClick;
    btnNext.OnClick := DoNextClick;
    btnCancel.OnClick := DoCancelClick;
    FPageIndex := 0;
    Pages[FPageIndex].Initialize;
  end;
  if Assigned(FOnInit) then FOnInit(Self);
end;

function TCustomWizard97.Execute: TExecuteResult;
begin
  Result := erCanceled;
  DoInitialize;
  FExecuting := TRUE;
  if wsOnTop in Style then
    WizardForm.FormStyle := fsStayOnTop;
  if wsModal in Style then
    WizardForm.ShowModal
  else
    WizardForm.Show;
end;

function TCustomWizard97.Executing: Boolean;
begin
  Result := FExecuting;
end;

procedure TCustomWizard97.NextPage;
begin
  Inc(FPageIndex);
  FPages[FPageIndex].Initialize;
end;

procedure TCustomWizard97.PrevPage;
begin
  Dec(FPageIndex);
  FPages[FPageIndex].Initialize;
end;

procedure TCustomWizard97.SetCaption(Value: TCaption);
begin
  if FFormCaption <> Value then
  begin
    FFormCaption := Value;
    ValidateString(string(FFormCaption));
    WizardForm.Caption := Format('%s Wizard', [FFormCaption]);
  end;
end;

procedure TCustomWizard97.SetSymbolGraphic(Value: TBitmap);
begin
  FSymbolGraphic.Assign(Value);
end;

procedure TCustomWizard97.SetWizardName(Value: string);
begin
  if FWizardName <> Value then
    FWizardName := Value;
  ValidateString(FWizardName);
end;

function TCustomWizard97.WizardForm: TWizard97Form;
begin
  if not Assigned(FWizard97Form) then
    FWizard97Form := TWizard97Form.Create(Self);
  Result := FWizard97Form;
end;

end.
